package com.citizen.app.posapplicationen;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;

/**
 * A class that can display a dialog containing a {@link android.widget.ProgressBar} and a {@link TextView}.
 * - Note that this class does not support terminal rotation.
 *
 * <p>(C) 2023 CITIZEN SYSTEMS JAPAN CO., LTD. All Rights Reserved.
 */
public class ProgressDialog extends Activity {
	private final AlertDialog mDialog;
	private final TextView mDialogMessage;

	public ProgressDialog(@NonNull Context context) {
		AlertDialog.Builder builder = new AlertDialog.Builder(context);
		View dialogView = View.inflate(context, R.layout.layout_progress_dialog, null);
		builder.setView(dialogView);
		mDialog = builder.create();
		mDialog.setCancelable(false);
		mDialog.setCanceledOnTouchOutside(false);
		mDialogMessage = dialogView.findViewById(R.id.progressDialogMessage);
	}

	/** Show progress dialog. */
	public synchronized void show(String message) {
		runOnUiThread(() -> {
			mDialogMessage.setText(message);
			mDialog.show();
		});
	}

	/** Dismiss progress dialog. */
	public synchronized void dismiss() {
		runOnUiThread(() -> {
			if (mDialog.isShowing()) {
				mDialog.dismiss();
			}
		});
	}

}
